﻿#Region "File Header"
'-----------------------------------------------------------------------
' File Name         :   AddCustomControls.vb
' File Description  :   1.Adding Panel Control
'                       2.Adding TableLayoutPanel Control
'                       3.Adding Custom Button Control
'
' Date Created      :   28th September 2012
' Date Modified     :   
'-----------------------------------------------------------------------
#End Region

#Region "Namespaces"
Imports System
Imports System.Windows.Forms
Imports System.Drawing
Imports SmartShopping.Data
#End Region

#Region "AddCustomControls Class"
Public Class AddCustomControls

#Region "Private Global Variables"
    Private _tlpBtnItems As TableLayoutPanel
#End Region

#Region "WinForm Layout"

    ''' <summary>
    ''' Function replaces Panel Controls
    ''' </summary>
    ''' <param name="pnlName"></param>
    ''' <remarks></remarks>
    Public Sub AddPanel(ByVal pnlName As String, ByVal parentForm As Form)

        Dim objPanelCtrl As Panel = Nothing
        Dim pnlMain As Panel = GetMainPanel(parentForm)
        Dim tlpControl As TableLayoutPanel = AddTableLayoutPanel(parentForm)

        If pnlName = "pnlDepartment" Then
            'No Panel to Hide
        ElseIf pnlName = "pnlCategory" Then
            'Hide Department Panel
            For Each ctrls As Control In pnlMain.Controls
                If ctrls.Name = "pnlDepartment" Then
                    ctrls.Hide()
                End If
            Next
        Else
            'Hide Category panel
            For Each ctrls As Control In pnlMain.Controls
                If ctrls.Name = "pnlCategory" Then
                    ctrls.Hide()
                End If
            Next
        End If

        objPanelCtrl = CustomizePanel(pnlName)
        objPanelCtrl.Controls.Add(tlpControl)
        pnlMain.Controls.Add(objPanelCtrl)

    End Sub

    ''' <summary>
    ''' Function Returns Custom Panel
    ''' </summary>
    ''' <param name="panelName"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function CustomizePanel(ByVal panelName As String) As Panel

        Dim customPanel As Panel = New Panel()
        customPanel.Name = panelName
        customPanel.BorderStyle = BorderStyle.FixedSingle
        customPanel.Dock = DockStyle.Fill
        customPanel.AutoScroll = True
        'customPanel.ContextMenuStrip = AddContextMenuStrip()

        CustomizePanel = customPanel

    End Function

    '''' <summary>
    '''' Function adds Context Menu Controls to Tool Strip Menu
    '''' </summary>
    '''' <returns></returns>
    '''' <remarks></remarks>
    'Private Function AddContextMenuStrip() As ContextMenuStrip

    '    Dim contextMenuStrip As New ContextMenuStrip()

    '    Dim tsMenuItemAddNewButton As New ToolStripMenuItem()
    '    tsMenuItemAddNewButton.Text = "Add Button"
    '    AddHandler tsMenuItemAddNewButton.Click, AddressOf tsMenuItemAddNewButton_Click

    '    contextMenuStrip.Items.Add(tsMenuItemAddNewButton)
    '    AddContextMenuStrip = contextMenuStrip

    'End Function

    '''' <summary>
    '''' Function Handles Tool Strip Item Click Event
    '''' </summary>
    '''' <param name="sender"></param>
    '''' <param name="e"></param>
    '''' <remarks></remarks>
    'Private Sub tsMenuItemAddNewButton_Click(ByVal sender As Object, ByVal e As EventArgs)

    '    Dim btnName As String = InputBox("Enter the Button Name")
    '    If Not btnName = "" Then

    '        Dim dynButton As New CustomButton(btnName, "1212")
    '        _tlpBtnItems.Controls.Add(dynButton.Button)

    '    End If

    'End Sub

    ''' <summary>
    ''' Function adds TableLayoutPanel
    ''' </summary>
    ''' <remarks></remarks>
    Private Function AddTableLayoutPanel(ByVal parentForm As Form) As TableLayoutPanel

        _tlpBtnItems = New TableLayoutPanel()
        _tlpBtnItems.Name = "tlpControl"
        _tlpBtnItems.AutoSize = True

        _tlpBtnItems.ColumnCount = 3
        _tlpBtnItems.ColumnStyles.Clear()
        _tlpBtnItems.ColumnStyles.Add(New ColumnStyle(SizeType.Percent, 33.33F))
        _tlpBtnItems.ColumnStyles.Add(New ColumnStyle(SizeType.Percent, 33.33F))
        _tlpBtnItems.ColumnStyles.Add(New ColumnStyle(SizeType.Percent, 33.33F))

        Return _tlpBtnItems

    End Function

    ''' <summary>
    ''' Function Returns WinForm TabPage Control
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetMainPanel(ByVal winForm As Form) As Panel

        Dim mainPanel As Panel = Nothing

        Try
            'Searching Main Panel
            For Each winformCtrl As Control In winForm.Controls
                For Each scCtrls As Control In winformCtrl.Controls
                    For Each tlpCtrls As Control In scCtrls.Controls
                        If tlpCtrls.Name = "tlpLeft" Then
                            For Each pnlCtrls As Control In tlpCtrls.Controls
                                If pnlCtrls.Name = "pnlMain" Then
                                    mainPanel = pnlCtrls
                                End If
                            Next
                        End If
                    Next
                Next
            Next

            GetMainPanel = mainPanel

        Catch ex As Exception
            MsgBox("Unable to locate Panel Control")
            GetMainPanel = mainPanel
        End Try

    End Function

#End Region

#Region "Add Item Buttons"

    ''' <summary>
    ''' Function adds Scrolling Text Lable Control
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ScrollingLabelCtrl() As Label

        Dim lblText As New Label()
        lblText.Font = New Font("Segoe Print", 12, FontStyle.Regular)
        lblText.ForeColor = Color.Red
        lblText.Text = DbOperations.LatestInventoryItem()
        lblText.AutoSize = True

        Return lblText

    End Function

    ''' <summary>
    ''' Function Populates Buttons
    ''' </summary>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <remarks></remarks>
    Public Sub PopulateButtons(Optional ByVal departmentId As String = Nothing, _
                               Optional ByVal categoryId As String = Nothing)

        'If Department Id Passed then Populate Categories
        'Else If Category Id And Department Id Passed then Populate items
        'Else Populate Departments
        If Not departmentId Is Nothing And categoryId Is Nothing Then

            AddCategoryButtons(departmentId)

        ElseIf Not departmentId Is Nothing And Not categoryId Is Nothing Then

            AddItemButtons(departmentId, categoryId)

        Else
            AddDepartmentButtons()

        End If

    End Sub

    ''' <summary>
    ''' Function adds Departments
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AddDepartmentButtons()

        Dim dtDepartment As New DataTable("Department")
        DepartmentSearchList = New AutoCompleteStringCollection()

        'Fetch Department Table from Db using DbOperations
        dtDepartment = DbOperations.GetAllDepartments()

        For Each dr As DataRow In dtDepartment.Rows
            'Adding Controls Names to a List
            DepartmentSearchList.Add(dr("Name").ToString())

            Dim dynButton As New CustomButton(dr("Name"), dr("ID"))
            _tlpBtnItems.Controls.Add(dynButton.Button)
        Next

    End Sub

    ''' <summary>
    ''' Functions adds Categories
    ''' </summary>
    ''' <param name="departmentId"></param>
    ''' <remarks></remarks>
    Private Sub AddCategoryButtons(ByVal departmentId As String)

        Dim dtCategory As New DataTable("Category")
        CategorySearchList = New AutoCompleteStringCollection()

        'Fetch Category Tablet from Db using DbOperations
        dtCategory = DbOperations.GetAllCategories(departmentId)

        For Each dr As DataRow In dtCategory.Rows
            'Adding Controls Name to a List
            CategorySearchList.Add(dr("Name").ToString())

            Dim dynButton As New CustomButton(dr("Name"), dr("DepartmentID"), dr("ID"))
            _tlpBtnItems.Controls.Add(dynButton.Button)
        Next

    End Sub

    ''' <summary>
    ''' Function adds Items
    ''' </summary>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <remarks></remarks>
    Private Sub AddItemButtons(ByVal departmentId As String, ByVal categoryId As String)

        Dim dtItems As New DataTable("Item")
        ItemSearchList = New AutoCompleteStringCollection()

        'Fetch Item Table from Db using DbOperations
        dtItems = DbOperations.GetAllItems(departmentId, categoryId)

        For Each dr As DataRow In dtItems.Rows
            'Adding Controls Names to a List
            ItemSearchList.Add(dr("ItemLookUpCode").ToString())

            Dim dynButton As New CustomButton(dr("Description"), dr("DepartmentID"), dr("CategoryID"), dr("ID"), dr("ItemLookUpCode"))
            _tlpBtnItems.Controls.Add(dynButton.Button)
        Next

    End Sub

#End Region

End Class
#End Region