﻿#Region "File Header"
' ----------------------------------------------------------------------------------------
' File Name          :   ApplicationLogs.vb
' File Description   :   1.Exception Handling Logs
'                        2.Application Event Logs
' Date Created       :   16th October 2012
' Date Modified      :   
' ----------------------------------------------------------------------------------------
#End Region

#Region "Namespaces"
Imports System.IO
#End Region

#Region "ApplicationLogs"
''' <summary>
''' Class used for Logging Events and Exceptions
''' </summary>
''' <remarks></remarks>
Public Class ApplicationLogs

#Region "Private Constant Variable"
    Private Const ERROR_FOLDER_NAME As String = "Errors"
#End Region

#Region "Public Constant Variables"
    Public Const LOG_FILE_NAME As String = "Errors\ErrorLog.txt"
    Public Const EVENTLOG_APPLICATIONNAME As String = "SmartShopping"
    Public Const EVENTLOG_LOGNAME As String = "DynButtons"
    Public Const EVENTLOG_SOURCE As String = EVENTLOG_APPLICATIONNAME & " System"
    Public Const Threading_SleepWaitSeconds As Integer = 5
#End Region

#Region "Methods"

    ''' <summary>
    ''' To write the Error Message Informationi in the ErrorLog.txt File
    ''' </summary>
    ''' <param name="SourceApplication"></param>
    ''' <param name="Message"></param>
    ''' <remarks></remarks>
    Public Shared Sub WriteTextLog(ByVal SourceApplication As String, ByVal Message As String)

        Dim sTime As String = DateTime.Now.ToString
        Dim sContent As String = sTime & " : " & SourceApplication & " : " & Message
        Dim aPath As String = System.AppDomain.CurrentDomain.BaseDirectory

        Try
            '   Checking for "Error" folder is exist or not in application Debug folder.
            If Not System.IO.Directory.Exists(aPath & ERROR_FOLDER_NAME) Then
                '   If not then Create Directory inside it.
                System.IO.Directory.CreateDirectory(aPath & ERROR_FOLDER_NAME)

            End If

            '   Checking File "ErrorLog.txt" is exist or not
            If Not System.IO.File.Exists(aPath & LOG_FILE_NAME) Then
                '   If Not then Creating a File.
                Dim objFStream As FileStream = New FileStream(aPath & LOG_FILE_NAME, FileMode.Create, FileAccess.ReadWrite)
                Dim objStream As StreamWriter = New StreamWriter(objFStream)

                objStream.Close()
                objFStream.Close()

            End If

            '   Open a file for Append information.
            Dim oFStream As FileStream = New FileStream(aPath & LOG_FILE_NAME, FileMode.Append, FileAccess.Write)
            Dim oStream As StreamWriter = New StreamWriter(oFStream)

            '   Write Content into the ErrorLog.txt file.
            oStream.Write(sContent & vbCrLf & vbCrLf)

            oStream.Close()
            oFStream.Close()

        Catch fileEx As FileNotFoundException
            '   Write File Exception message into the Event Log file.
            ApplicationLogs.WriteEventLog("ErrorLog.WriteTextLog()", fileEx.Message, Diagnostics.EventLogEntryType.Error)

        Catch ex As Exception
            '   Write Exception message into the Event Log file.
            ApplicationLogs.WriteEventLog("ErrorLog.WriteTextLog()", ex.Message, Diagnostics.EventLogEntryType.Error)

        End Try

    End Sub

    ''' <summary>
    '''  Write the Error Message in Event Viewer Log file.
    ''' </summary>
    ''' <param name="SourceFile"></param>
    ''' <param name="Message"></param>
    ''' <param name="EventLogEntryType"></param>
    ''' <remarks></remarks>
    Public Shared Sub WriteEventLog(ByVal SourceFile As String, ByVal Message As String, _
                                    ByVal EventLogEntryType As System.Diagnostics.EventLogEntryType)

        Try
            Dim eLog As System.Diagnostics.EventLog

            If Not Diagnostics.EventLog.Exists(EVENTLOG_LOGNAME) Then
                '   Create LOG file in Event Viewer by calling this function.
                CreateEventLog()

            End If

            '   Select Event name.
            eLog = New System.Diagnostics.EventLog(EVENTLOG_LOGNAME)

            '   Set the Source name for the message.
            eLog.Source = EVENTLOG_SOURCE

            '   Writing the message into the LOG file.
            eLog.WriteEntry(SourceFile & ":  " & Message, EventLogEntryType)

            '   Close the event.
            eLog.Close()

        Catch ex As Exception
            '   Write the Exception message into the ErrorLog.txt file.
            ApplicationLogs.WriteTextLog("ErrorLog.WriteEventLog()", ex.Message)

        End Try

    End Sub

    ''' <summary>
    ''' Create Event in EventLogName
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared Sub CreateEventLog()

        Try

            '   Create Event into the Event Viewer.
            Diagnostics.EventLog.CreateEventSource(EVENTLOG_SOURCE, EVENTLOG_LOGNAME)
            System.Threading.Thread.Sleep(System.TimeSpan.FromSeconds(Threading_SleepWaitSeconds))

        Catch ex As Exception

            '   Write Exception message into the ErrorLog.txt file.
            ApplicationLogs.WriteTextLog("ErrorLog.CreateEventLog()", ex.Message)

        End Try

    End Sub

#End Region

End Class
#End Region