﻿#Region "FileHeader"
'-----------------------------------------------------------------------
' File Name         :   CustomButton.vb
' File Description  :   1.Create Custom Button
'                       2.Add Buttons to WinForm
'                       3.Remove Buttons from WinForm
'
' Date Created      :   28th September 2012
' Date Modified     :
'-----------------------------------------------------------------------
#End Region

#Region "Namespaces"
Imports System
Imports System.Windows.Forms
Imports System.Drawing
Imports SmartShopping.Data
Imports System.IO
#End Region

#Region "CustomButton Class"
Public Class CustomButton

#Region "Private Variables"
    Private _button As Button
    Private _name As String
    Private _imgPath As String

    Private _messageTimer As Timer = New Timer()
    Private _progressTimer As Timer = New Timer()

    Private _deptId As Nullable(Of Integer)
    Private _cateId As Nullable(Of Integer)
    Private _itemId As Nullable(Of Integer)
#End Region

#Region "Public Variable"
    Dim objButtonCtrl As Button = New Button()
#End Region

#Region "Public Properties"
    ''' <summary>
    ''' Property for Button Control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Button() As Button
        Get
            Return _button
        End Get
        Set(ByVal value As Button)
            _button = value
        End Set
    End Property

    ''' <summary>
    ''' Property for Button Name
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            _name = value
        End Set
    End Property

    ''' <summary>
    ''' Property for Button Background Image
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property ImagePath() As String
        Get
            Return _imgPath
        End Get
        Set(ByVal value As String)
            _imgPath = value
        End Set
    End Property

    ''' <summary>
    ''' Property for Items Department ID
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property DepartmentID() As Integer
        Get
            Return _deptId
        End Get
        Set(ByVal value As Integer)
            _deptId = value
        End Set
    End Property

    ''' <summary>
    ''' Property for Items Category ID
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property CategoryID() As Integer
        Get
            Return _cateId
        End Get
        Set(ByVal value As Integer)
            _cateId = value
        End Set
    End Property

    ''' <summary>
    ''' Property for Items Item ID
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property ItemID() As Integer
        Get
            Return _itemId
        End Get
        Set(ByVal value As Integer)
            _itemId = value
        End Set
    End Property
#End Region

    ''' <summary>
    ''' Function for Parametrised Constructor
    ''' </summary>
    ''' <param name="name"> Name Text </param>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <param name="itemId"></param>
    ''' <remarks></remarks>
    Public Sub New(ByVal name As String, Optional ByVal departmentId As String = Nothing, _
                   Optional ByVal categoryId As String = Nothing, Optional ByVal itemId As String = Nothing, _
                   Optional ByVal itemLookUpCode As String = Nothing)

        Me.Name = name.Trim()

        If departmentId Is Nothing Then
            Me.DepartmentID = 0
        Else
            Me.DepartmentID = Convert.ToInt16(departmentId)
        End If

        If categoryId Is Nothing Then
            Me.CategoryID = 0
        Else
            Me.CategoryID = Convert.ToInt16(categoryId)
        End If

        If itemId Is Nothing Then
            Me.ItemID = 0
        Else
            Me.ItemID = Convert.ToInt16(itemId)
        End If

        Me.Button = CreateButton(itemLookUpCode)

    End Sub

    ''' <summary>
    ''' Function Creates Custom Button
    ''' </summary>
    ''' <param name="itemLookUpCode"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function CreateButton(ByVal itemLookUpCode As String) As Button

        objButtonCtrl.Name = itemLookUpCode
        objButtonCtrl.Font = New Font("Calibri", 10, FontStyle.Bold)
        objButtonCtrl.Text = Name.ToUpper()
        objButtonCtrl.ForeColor = Color.White
        objButtonCtrl.TextAlign = ContentAlignment.MiddleLeft
        objButtonCtrl.Anchor = AnchorStyles.None
        objButtonCtrl.Size = New Size(252, 60)
        objButtonCtrl.FlatStyle = FlatStyle.Popup
        objButtonCtrl.BackColor = Color.Transparent

        If Not (GetButtonImage() = String.Empty) Then
            objButtonCtrl.Image = New Bitmap(GetButtonImage())
        End If

        objButtonCtrl.ImageAlign = ContentAlignment.MiddleRight
        objButtonCtrl.ContextMenuStrip = AddContextMenuStrip()

        If Not ItemID = Nothing Then

            Dim lblButtonText As Label = New Label()

            lblButtonText.ForeColor = Color.Black
            lblButtonText.Font = New Font("Calibri", 12, FontStyle.Bold)
            lblButtonText.TextAlign = ContentAlignment.TopCenter
            lblButtonText.AutoSize = True

            lblButtonText.Text = "|||||" & itemLookUpCode & "|||||"

            objButtonCtrl.Font = New Font("Calibri", 10, FontStyle.Bold)
            objButtonCtrl.TextAlign = ContentAlignment.BottomLeft
            objButtonCtrl.Text = Name.ToUpper()

            objButtonCtrl.Controls.Add(lblButtonText)

            Dim objToolTip As ToolTip = New ToolTip()
            objToolTip.ToolTipTitle = "Item Description"
            objToolTip.ToolTipIcon = ToolTipIcon.Info
            objToolTip.IsBalloon = True
            objToolTip.OwnerDraw = True
            objToolTip.BackColor = Color.Navy
            objToolTip.ForeColor = Color.White
            objToolTip.AutoPopDelay = 5000
            objToolTip.InitialDelay = 100
            objToolTip.ReshowDelay = 100
            Dim itemInfo As String = DbOperations.GetDetailItemInfoByItemID(ItemID)

            objToolTip.SetToolTip(objButtonCtrl, itemInfo)
        End If

        AddHandler objButtonCtrl.Click, AddressOf objButtonCtrl_Click

        Return objButtonCtrl

    End Function

    ''' <summary>
    ''' Function Adds New ContextMenuStrip with ToolStripItems
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function AddContextMenuStrip() As ContextMenuStrip

        Dim buttonContextMenuStrip As New ContextMenuStrip()

        'Dim tsItemRemoveButton As ToolStripItem = buttonContextMenuStrip.Items.Add("Remove Item")
        'AddHandler tsItemRemoveButton.Click, AddressOf tsItemRemoveButton_Click

        Dim tsItemEditButton As ToolStripItem = buttonContextMenuStrip.Items.Add("Edit Name")
        AddHandler tsItemEditButton.Click, AddressOf tsItemEditButton_Click

        If ItemID > 0 Then
            Dim tsItemAddEditPicture As ToolStripItem = buttonContextMenuStrip.Items.Add("Add or Update Item Picture")
            AddHandler tsItemAddEditPicture.Click, AddressOf tsItemAddEditPicture_Click

            Dim tsItemRemovePicture As ToolStripItem = buttonContextMenuStrip.Items.Add("Remove Item Picture")
            AddHandler tsItemRemovePicture.Click, AddressOf tsItemRemovePicture_Click
        End If

        AddContextMenuStrip = buttonContextMenuStrip

    End Function

    '''' <summary>
    '''' Function Handles Tool Strip Item Click Event
    '''' </summary>
    '''' <param name="sender"></param>
    '''' <param name="e"></param>
    '''' <remarks></remarks>
    'Private Sub tsItemRemoveButton_Click(ByVal sender As Object, ByVal e As EventArgs)

    'End Sub

    ''' <summary>
    ''' Function Handles Tool Strip Item Click Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsItemEditButton_Click(ByVal sender As Object, ByVal e As EventArgs)

        Dim buttonName As String = objButtonCtrl.Text.ToString()
        Dim buttonRename As String = Interaction.InputBox("Enter the New Name for" & vbNewLine & buttonName, "SmartShopping | Rename", buttonName)

        If Not buttonRename = "" Then
            Me.Name = buttonRename.Trim()
            objButtonCtrl.Text = buttonRename.ToUpper()
            RenameButton(buttonRename)
            ModifyNamesList(buttonName, buttonRename)
        End If

    End Sub

    ''' <summary>
    ''' Function Handles Tool Strip Item Click Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsItemAddEditPicture_Click(ByVal sender As Object, ByVal e As EventArgs)

        Dim imageName As String = String.Empty
        Dim imagePath As String = String.Empty
        Dim openFileDailog As New OpenFileDialog()

        openFileDailog.Title = "Enter the Image Path"
        openFileDailog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop)
        openFileDailog.Filter = "Images (*.png,*.bmp,*.jpeg)|*.png;*.bmp;*.jpeg"
        openFileDailog.FilterIndex = 1

        If (openFileDailog.ShowDialog = DialogResult.OK) Then

            imageName = Path.GetFileName(openFileDailog.FileName())
            imagePath = Path.GetFullPath(openFileDailog.FileName())

            If objButtonCtrl.Image Is Nothing Then
                objButtonCtrl.Image = New Bitmap(openFileDailog.FileName())
                DbOperations.AddStoreImageInfo(DepartmentID, CategoryID, ItemID, imageName, imagePath)
            Else
                objButtonCtrl.Image = New Bitmap(openFileDailog.FileName())
                DbOperations.ModifyStoreImageInfo(DepartmentID, CategoryID, ItemID, imageName, imagePath)
            End If
            

        End If

    End Sub

    ''' <summary>
    ''' Function Handles Tool Strip Item Click Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsItemRemovePicture_Click(ByVal sender As Object, ByVal e As EventArgs)
        objButtonCtrl.Image = Nothing
        DbOperations.DeleteStoreImageInfo(DepartmentID, CategoryID, ItemID)
    End Sub

    ''' <summary>
    ''' Function Handles Dynamic Button Click Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub objButtonCtrl_Click(ByVal sender As Object, ByVal e As EventArgs)

        'Initializing the Message Timer
        _messageTimer.Interval = 1500
        AddHandler _messageTimer.Tick, AddressOf MessageTimer_Tick

        'Initializing the Progress Timer
        _progressTimer.Interval = 2000
        AddHandler _progressTimer.Tick, AddressOf ProgressTimer_Tick

        CType(FindTheControl("txtSearch"), TextBox).Clear()
        CType(FindTheControl("txtSearch"), TextBox).Focus()
        Dim addDynCtrls As New AddCustomControls()

        'If DepartmentID is available then Populate its Corresponding Categories
        'Else If Department and Category ID available then Populates their Corresponding Items
        If DepartmentID > 0 And CategoryID = Nothing And ItemID = Nothing Then

            CType(FindTheControl("lblItemType"), Label).Text = CType(sender, Button).Text

            CType(FindTheControl("pnlProgress"), Panel).Visible = True
            _progressTimer.Start()

            addDynCtrls.AddPanel("pnlCategory", WinFormItems.ActiveForm)
            'Calling to Populate Categories
            addDynCtrls.PopulateButtons(DepartmentID, Nothing)

            CType(FindTheControl("lblFooter"), Label).Text = "Category List"

            CType(FindTheControl("txtSearch"), TextBox).AutoCompleteCustomSource = CategorySearchList

        ElseIf DepartmentID > 0 And CategoryID > 0 And ItemID = Nothing Then

            CType(FindTheControl("lblItemType"), Label).Text += " > " & CType(sender, Button).Text

            CType(FindTheControl("pnlProgress"), Panel).Visible = True
            _progressTimer.Start()

            addDynCtrls.AddPanel("pnlItem", WinFormItems.ActiveForm)
            'Calling to Populate Items
            addDynCtrls.PopulateButtons(DepartmentID, CategoryID)

            CType(FindTheControl("lblQuantity"), Label).Visible = True
            CType(FindTheControl("nudQuantity"), NumericUpDown).Value = 1
            CType(FindTheControl("nudQuantity"), NumericUpDown).Visible = True

            'CType(FindTheControl("btnAdd"), Button).Enabled = True
            CType(FindTheControl("lblFooter"), Label).Text = "Item List"

            CType(FindTheControl("txtSearch"), TextBox).AutoCompleteCustomSource = ItemSearchList

        Else
            'CType(FindTheControl("nudQuantity"), NumericUpDown).Value = 1
            'CType(FindTheControl("nudQuantity"), NumericUpDown).Focus()

            CType(FindTheControl("txtSearch"), TextBox).AutoCompleteCustomSource = ItemSearchList
            Dim itemLookUpCode As String = objButtonCtrl.Name

            Dim quantityPurchased As Decimal
            If (Not Decimal.TryParse(CType(FindTheControl("nudQuantity"), NumericUpDown).Controls.Item(1).Text, quantityPurchased)) Or (Convert.ToInt16(quantityPurchased) < 1) Then
                MsgBox("Please specify the Quantity", MsgBoxStyle.Exclamation)
                CType(FindTheControl("nudQuantity"), NumericUpDown).Select()
            ElseIf quantityPurchased > DbOperations.GetItemOnHandQuantityByItemId(ItemID) Then
                MsgBox("Please specify a Quantity" & vbNewLine & "As per On Hand Quantity availability", MsgBoxStyle.Exclamation)
                CType(FindTheControl("nudQuantity"), NumericUpDown).Select()
            Else
                Dim dtItemInfo As New DataTable()

                If Not itemLookUpCode = String.Empty Then
                    dtItemInfo = DbOperations.GetItemInfoByItemLookUpCode(itemLookUpCode)
                End If

                If Not dtItemInfo Is Nothing AndAlso dtItemInfo.Rows.Count <> 0 Then
                    For Each dr As DataRow In dtItemInfo.Rows
                        Dim objDataRow As DataRow = GetItemInfoTable()

                        objDataRow("ID") = dr("ID")
                        objDataRow("ItemLookUpCode") = dr("ItemLookUpCode")
                        objDataRow("QuantityPurchased") = quantityPurchased
                        objDataRow("Quantity") = dr("Quantity")
                        objDataRow("Price") = dr("Price")

                        AddInEntry.ItemRow = objDataRow
                    Next
                End If

                objButtonCtrl.ForeColor = Color.Green
                objButtonCtrl.Text = "Successfully Added"
                _messageTimer.Start()

            End If ' End of NumericData validation

        End If ' End of Button Click Validation

        'Populate Search TextBox's Search Texts
        CType(FindTheControl("txtSearch"), TextBox).AutoCompleteMode = AutoCompleteMode.SuggestAppend
        CType(FindTheControl("txtSearch"), TextBox).AutoCompleteSource = AutoCompleteSource.CustomSource

        CType(FindTheControl("btnBack"), Button).Visible = True
        CType(FindTheControl("btnHome"), Button).Visible = True

    End Sub

    ''' <summary>
    ''' Function Returns the Specific Control
    ''' </summary>
    ''' <param name="name"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function FindTheControl(ByVal name As String) As Control

        Dim winFormItems As Form = Application.OpenForms("WinFormItems")
        Dim formCtrl As Control = Nothing

        For Each spCtrl As Control In winFormItems.Controls
            'Control Search in Panel 1
            For Each spPanel As Control In CType(spCtrl, SplitContainer).Panel1.Controls
                For Each tlpCtrl As Control In spPanel.Controls
                    If (tlpCtrl.Name = "tlpHeader") Or (tlpCtrl.Name = "pnlMain") Or (tlpCtrl.Name = name) Then

                        If tlpCtrl.Name = name Then
                            formCtrl = CType(tlpCtrl, Label)
                        End If

                        For Each ctrl In tlpCtrl.Controls
                            If ctrl.Name = name Then
                                formCtrl = ctrl
                            End If
                        Next

                    End If
                Next
            Next
        Next

        If formCtrl Is Nothing Then
            'Control Search in Panel 2
            For Each spCtrl As Control In winFormItems.Controls
                For Each spPanel As Control In CType(spCtrl, SplitContainer).Panel2.Controls
                    For Each ctrl As Control In spPanel.Controls
                        If ctrl.Name = name Then
                            formCtrl = ctrl
                        End If
                    Next
                Next
            Next

            Return formCtrl
        Else
            Return formCtrl
        End If

    End Function

    ''' <summary>
    ''' Function Returns a DataRow for ItemsInfo
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetItemInfoTable() As DataRow

        Dim dtItems As DataTable = New DataTable

        dtItems.Columns.Add("ID", Type.GetType("System.Int32"))
        dtItems.Columns.Add("ItemLookUpCode", Type.GetType("System.String"))
        dtItems.Columns.Add("QuantityPurchased", Type.GetType("System.Double"))
        dtItems.Columns.Add("Quantity", Type.GetType("System.Double"))
        dtItems.Columns.Add("Price", Type.GetType("System.Decimal"))

        Return dtItems.NewRow()

    End Function

    ''' <summary>
    ''' Functions Handles the Message Timer Tick Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MessageTimer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        _messageTimer.Stop()
        objButtonCtrl.ForeColor = Color.White
        'objButtonCtrl.Enabled = True
        objButtonCtrl.Text = Name.ToUpper
    End Sub

    ''' <summary>
    ''' Functions Handles the Progress Timer Tick Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ProgressTimer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        _progressTimer.Stop()
        CType(FindTheControl("pnlProgress"), Panel).Visible = False
    End Sub

    ''' <summary>
    ''' Function to Rename Buttons
    ''' </summary>
    ''' <param name="strName"></param>
    ''' <remarks></remarks>
    Private Sub RenameButton(ByVal strName As String)

        'If DepartmentID is available then Rename its Corresponding Categories
        'Else If Department and Category ID available then Rename their Corresponding Items
        'Else Rename Department
        If DepartmentID > 0 And CategoryID = Nothing And ItemID = Nothing Then
            DbOperations.ModifyName(strName, DepartmentID, 0, 0)
        ElseIf DepartmentID > 0 And CategoryID > 0 And ItemID = Nothing Then
            DbOperations.ModifyName(strName, DepartmentID, CategoryID, 0)
        ElseIf DepartmentID > 0 And CategoryID > 0 And ItemID > 0 Then
            DbOperations.ModifyName(strName, Nothing, Nothing, ItemID)
        End If

    End Sub

    ''' <summary>
    ''' Function Returns Image File Path
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetButtonImage() As String

        Dim btnImagePath As String = String.Empty

        Try
            btnImagePath = DbOperations.GetStoreImageInfo(DepartmentID, CategoryID, ItemID)
            If File.Exists(btnImagePath) Then
                Return btnImagePath
            Else
                Return String.Empty
            End If
        Catch fileEx As FileNotFoundException
            ErrorLog.WriteTextLog(fileEx.Message + " -- Image File Not Found", fileEx.StackTrace)
            Return btnImagePath
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message, ex.StackTrace)
            Return btnImagePath
        End Try

    End Function

    ''' <summary>
    ''' Function Modify the Names List with New Names
    ''' </summary>
    ''' <param name="oldName"></param>
    ''' <param name="newName"></param>
    ''' <remarks></remarks>
    Private Sub ModifyNamesList(ByVal oldName As String, ByVal newName As String)

        If DepartmentID > 0 And CategoryID = 0 And ItemID = 0 Then

            For Each strName As String In DepartmentSearchList
                If strName = oldName Then
                    DepartmentSearchList.Remove(strName)
                    DepartmentSearchList.Add(newName)
                    Exit For
                End If
            Next

        ElseIf DepartmentID > 0 And CategoryID > 0 And ItemID = 0 Then

            For Each strName As String In CategorySearchList
                If strName = oldName Then
                    CategorySearchList.Remove(strName)
                    CategorySearchList.Add(newName)
                    Exit For
                End If
            Next

        ElseIf DepartmentID > 0 And CategoryID > 0 And ItemID > 0 Then

            For Each strName As String In ItemSearchList
                If strName = oldName Then
                    ItemSearchList.Remove(strName)
                    ItemSearchList.Add(newName)
                    Exit For
                End If
            Next

        End If

    End Sub

End Class
#End Region