﻿#Region "File Header"
'-----------------------------------------------------------------------
' File Name         :   DbOperations.vb
' File Description  :   1.Custom POS Button Entry
'                       2.Getting Depatments, Categories, Items
'                       3.Modifying Items
'                       4.Delete Items
'                       5.Add Departments, Categories, Items
'
' Date Created      :   28th September 2012
' Date Modified     :   21st December 2012
'-----------------------------------------------------------------------
#End Region

#Region "Namespaces"
Imports System
Imports System.Data.Sql
Imports System.Configuration
Imports System.Data.SqlClient
Imports System.Text
#End Region

#Region "DbOperations Class"
Public Class DbOperations

    ''' <summary>
    ''' Property for Connection String
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared ReadOnly Property GetConnectionString() As String
        Get
            Return gConnectionString
        End Get
    End Property

    ''' <summary>
    ''' Function Creates Store Images Table
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function CreateStoreImagesTable() As Boolean

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()

                Using sqlCmd As New SqlCommand(Query.createStoreImagesTable, connection)
                    sqlCmd.ExecuteNonQuery()
                End Using

                connection.Close()
            End Using

            Return True
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Unable to Create Table", ex.StackTrace)
            Return False
        End Try

    End Function

    ''' <summary>
    ''' Function Insert Store Image Info into Database
    ''' </summary>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <param name="itemId"></param>
    ''' <param name="imageName"></param>
    ''' <param name="imagePath"></param>
    ''' <remarks></remarks>
    Public Shared Sub AddStoreImageInfo(ByVal departmentId As Nullable(Of Integer), _
                                        ByVal categoryId As Nullable(Of Integer), _
                                        ByVal itemId As Nullable(Of Integer), _
                                        ByVal imageName As String, ByVal imagePath As String)

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()

                Using sqlCmd As New SqlCommand(Query.insertStoreImageInfo, connection)
                    sqlCmd.Parameters.AddWithValue("@departmentId", departmentId)
                    sqlCmd.Parameters.AddWithValue("@categoryId", categoryId)
                    sqlCmd.Parameters.AddWithValue("@itemId", itemId)
                    sqlCmd.Parameters.AddWithValue("@imageName", imageName)
                    sqlCmd.Parameters.AddWithValue("@imagePath", imagePath)
                    sqlCmd.CommandType = CommandType.Text
                    sqlCmd.ExecuteNonQuery()
                End Using

                connection.Close()
            End Using
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Unable to Insert Image Info", ex.StackTrace)
        End Try


    End Sub

    ''' <summary>
    ''' Function Retrieves Store Image Info from Database
    ''' </summary>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <param name="itemId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetStoreImageInfo(ByVal departmentId As Nullable(Of Integer), _
                                             ByVal categoryId As Nullable(Of Integer), _
                                             ByVal itemId As Nullable(Of Integer)) As String

        Dim imagePath As String = String.Empty

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()

                If (departmentId > 0) And (categoryId > 0) And (itemId > 0) Then
                    Using sqlCmd As New SqlCommand(Query.selectItemImage, connection)
                        sqlCmd.Parameters.AddWithValue("@departmentId", departmentId)
                        sqlCmd.Parameters.AddWithValue("@categoryId", categoryId)
                        sqlCmd.Parameters.AddWithValue("@itemId", itemId)
                        sqlCmd.CommandType = CommandType.Text
                        imagePath = sqlCmd.ExecuteScalar()
                    End Using
                End If

                connection.Close()
            End Using
            Return imagePath
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Unable to Retrieve Image Info", ex.StackTrace)
            Return imagePath
        End Try

    End Function

    ''' <summary>
    ''' Function to Update Store Image Info
    ''' </summary>
    ''' <param name="imageName"></param>
    ''' <param name="imagePath"></param>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <param name="itemId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function ModifyStoreImageInfo(ByVal departmentId As Nullable(Of Integer), _
                                                ByVal categoryId As Nullable(Of Integer), _
                                                ByVal itemId As Nullable(Of Integer), _
                                                ByVal imageName As String, ByVal imagePath As String) As Boolean

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()

                Using sqlCmd As New SqlCommand(Query.updateStoreImageInfo, connection)
                    sqlCmd.Parameters.AddWithValue("@departmentId", departmentId)
                    sqlCmd.Parameters.AddWithValue("@categoryId", categoryId)
                    sqlCmd.Parameters.AddWithValue("@itemId", itemId)
                    sqlCmd.Parameters.AddWithValue("@imageName", imageName)
                    sqlCmd.Parameters.AddWithValue("@imagePath", imagePath)
                    sqlCmd.CommandType = CommandType.Text
                    sqlCmd.ExecuteNonQuery()
                End Using

                connection.Close()
            End Using
            Return True
        Catch ex As Exception
            Return False
            ErrorLog.WriteTextLog(ex.Message & " - Unable to Upadte Image Info", ex.StackTrace)
        End Try

    End Function

    ''' <summary>
    ''' Function Deletes Store Image Info
    ''' </summary>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <param name="itemId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function DeleteStoreImageInfo(Optional ByVal departmentId As Integer = Nothing, _
                                         Optional ByVal categoryId As Integer = Nothing, _
                                         Optional ByVal itemId As Integer = Nothing) As Boolean

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlCmd As New SqlCommand(Query.deleteStoreImageInfo, connection)
                    sqlCmd.Parameters.AddWithValue("@departmentId", departmentId)
                    sqlCmd.Parameters.AddWithValue("@categoryId", categoryId)
                    sqlCmd.Parameters.AddWithValue("@itemId", itemId)
                    sqlCmd.CommandType = CommandType.Text
                    sqlCmd.ExecuteNonQuery()
                End Using
                connection.Close()
            End Using
            Return True
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Unable to Delete Image Info", ex.StackTrace)
            Return False
        End Try

    End Function

    ''' <summary>
    ''' Function Returns New Item
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function LatestInventoryItem() As String

        Dim dtLatestItem As New DataTable()
        Dim sbLatestItem As New StringBuilder()

        Try
            Using connection As New SqlConnection(GetConnectionString)
                connection.Open()
                Using SqlDataAdp As New SqlDataAdapter(Query.selectLatestItem, connection)
                    SqlDataAdp.Fill(dtLatestItem)
                End Using
                connection.Close()
            End Using

            If dtLatestItem IsNot Nothing AndAlso dtLatestItem.Rows.Count <> 0 Then
                For Each drItem As DataRow In dtLatestItem.Rows
                    sbLatestItem.Append("New Item Added : ")
                    sbLatestItem.Append(drItem("Item"))
                    sbLatestItem.Append(" In ")
                    sbLatestItem.Append(drItem("Department"))
                    sbLatestItem.Append(" Under ")
                    sbLatestItem.Append(drItem("Category"))
                Next
            Else
                sbLatestItem.Append("No New Item  ")
            End If

            Return sbLatestItem.ToString()
        Catch ex As Exception
            Return Nothing
            ErrorLog.WriteTextLog(ex.Message & " - Unable to Fetch Data", ex.StackTrace)
        End Try

    End Function

    ''' <summary>
    ''' Function Returns Item Info Using ItemLookUpCode
    ''' </summary>
    ''' <param name="itemLookUpCode"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetItemInfoByItemLookUpCode(ByVal itemLookUpCode As String) As DataTable

        Dim dtItemInfo As New DataTable()

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using SqlDataAdp As New SqlDataAdapter(Query.selectItemInfoByItemLookUpCode.Replace("@itemLookUpCode", itemLookUpCode), connection)
                    SqlDataAdp.Fill(dtItemInfo)
                End Using
                connection.Close()
            End Using
            Return dtItemInfo
        Catch ex As Exception
            MsgBox(ex.Message)
            ErrorLog.WriteTextLog(ex.Message, ex.StackTrace)
            Return Nothing
        End Try

    End Function

    ''' <summary>
    ''' Function Returns Detailed Item Info
    ''' </summary>
    ''' <param name="itemId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetDetailItemInfoByItemID(ByVal itemId As Integer) As String

        Dim sbItemInfo As StringBuilder = New StringBuilder()
        Dim sqlDataRead As SqlDataReader

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlCmd As New SqlCommand(Query.selectDetailItemInfoByItemId.Replace("@itemID", itemId), connection)
                    sqlDataRead = sqlCmd.ExecuteReader()
                End Using

                If sqlDataRead.HasRows Then
                    While (sqlDataRead.Read())
                        sbItemInfo.Append("|||||")
                        sbItemInfo.Append(sqlDataRead("Barcode"))
                        sbItemInfo.Append("|||||")
                        sbItemInfo.Append(vbNewLine)
                        sbItemInfo.Append("Supplier : " & sqlDataRead("Supplier"))
                        sbItemInfo.Append(vbNewLine)
                        sbItemInfo.Append("Item Tax : " & sqlDataRead("ItemTax"))
                        sbItemInfo.Append(vbNewLine)
                        sbItemInfo.Append("Cost : " & sqlDataRead("Cost"))
                        sbItemInfo.Append(vbNewLine)
                        sbItemInfo.Append("On Hand Quantity : " & sqlDataRead("OnHandQuantity"))
                        sbItemInfo.Append(vbNewLine)
                        sqlDataRead.NextResult()
                    End While
                Else
                    sbItemInfo.Append("Description N/A")
                End If
                connection.Close()
            End Using

            Return sbItemInfo.ToString()
        Catch ex As Exception
            Return sbItemInfo.ToString()
            ErrorLog.WriteTextLog(ex.Message, ex.StackTrace)
        End Try

    End Function

    ''' <summary>
    ''' Function Returns On Hand Quantity of a Item
    ''' </summary>
    ''' <param name="itemId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetItemOnHandQuantityByItemId(ByVal itemId As Integer) As Decimal

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlCmd As New SqlCommand(Query.selectItemOnHandQuantityByItemId, connection)
                    sqlCmd.Parameters.AddWithValue("@itemId", itemId)
                    sqlCmd.CommandType = CommandType.Text
                    Return sqlCmd.ExecuteScalar()
                End Using
                connection.Close()
            End Using
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Unable to Fetch On Hand Quantity", ex.StackTrace)
        End Try

    End Function

    ''' <summary>
    ''' Function Returns Department Table
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetAllDepartments() As DataTable

        Dim dtDepartment As New DataTable("Department")

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlDataAdp As New SqlDataAdapter(Query.selectAllDepartment, connection)
                    sqlDataAdp.Fill(dtDepartment)
                End Using
                connection.Close()
            End Using
            GetAllDepartments = dtDepartment
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Failed to Fetch Data", ex.StackTrace)
            GetAllDepartments = Nothing
        End Try

    End Function

    ''' <summary>
    ''' Function Returns Category Table
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetAllCategories(ByVal departmentId As Integer) As DataTable

        Dim dtCategory As New DataTable("Category")

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlDataAdp As New SqlDataAdapter(Query.selectAllCategories.Replace("@departmentId", departmentId), connection)
                    sqlDataAdp.Fill(dtCategory)
                End Using
                connection.Close()
            End Using

            GetAllCategories = dtCategory

        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Failed to Fetch Data", ex.StackTrace)
            GetAllCategories = Nothing
        End Try

    End Function

    ''' <summary>
    ''' Function Returns Item Table
    ''' </summary>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetAllItems(ByVal departmentId As Integer, ByVal categoryId As Integer) As DataTable

        Dim dtItem As New DataTable("Item")

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlDataAdp As New SqlDataAdapter(Query.selectAllItems.Replace("@departmentId", departmentId).Replace("@categoryId", categoryId), connection)
                    sqlDataAdp.Fill(dtItem)
                End Using
                connection.Close()
            End Using
            GetAllItems = dtItem
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Failed to Fetch Data", ex.StackTrace)
            GetAllItems = Nothing
        End Try

    End Function

    ''' <summary>
    ''' Function for Renaming Departments, Categories and Items
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="departmentId"></param>
    ''' <param name="categoryId"></param>
    ''' <param name="itemId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function ModifyName(ByVal name As String, _
                                      ByVal departmentId As Nullable(Of Integer), _
                                      ByVal categoryId As Nullable(Of Integer), _
                                      ByVal itemId As Nullable(Of Integer)) As Boolean

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                If departmentId > 0 And categoryId = 0 And itemId = 0 Then
                    Using sqlCmd As New SqlCommand(Query.updateDepartmentName, connection)
                        sqlCmd.Parameters.AddWithValue("@name", name)
                        sqlCmd.Parameters.AddWithValue("@departmentId", departmentId)
                        sqlCmd.CommandType = CommandType.Text
                        sqlCmd.ExecuteNonQuery()
                    End Using
                ElseIf departmentId > 0 And categoryId > 0 And itemId = 0 Then
                    Using sqlCmd As New SqlCommand(Query.updateCategoryName, connection)
                        sqlCmd.Parameters.AddWithValue("@name", name)
                        sqlCmd.Parameters.AddWithValue("@departmentId", departmentId)
                        sqlCmd.Parameters.AddWithValue("@categoryId", categoryId)
                        sqlCmd.CommandType = CommandType.Text
                        sqlCmd.ExecuteNonQuery()
                    End Using
                ElseIf departmentId > 0 And categoryId > 0 And itemId > 0 Then
                    Using sqlCmd As New SqlCommand(Query.updateItemDescription, connection)
                        sqlCmd.Parameters.AddWithValue("@name", name)
                        sqlCmd.Parameters.AddWithValue("@departmentId", departmentId)
                        sqlCmd.Parameters.AddWithValue("@categoryId", categoryId)
                        sqlCmd.Parameters.AddWithValue("@itemId", itemId)
                        sqlCmd.CommandType = CommandType.Text
                        sqlCmd.ExecuteNonQuery()
                    End Using
                End If
                connection.Close()
                Return True
            End Using
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Failed to Rename", ex.StackTrace)
            Return False
        End Try

    End Function

    ''' <summary>
    ''' Function Inserts Custom POS Button Details
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function InsertCustomButtonDetails() As Boolean

        Dim maxNumber As Integer = 0
        maxNumber = GetMaxNumberOfPOS()

        If maxNumber = 999 Then
            MsgBox("Max POS Entry is " + maxNumber.ToString())
            Exit Function

        Else
            Try
                Using connection As New SqlConnection(GetConnectionString())
                    connection.Open()
                    Using sqlCmd As New SqlCommand(Query.insertCustomPOS.Replace("@maxNumber", maxNumber), connection)
                        sqlCmd.ExecuteNonQuery()
                    End Using
                    connection.Close()
                End Using
                Return True

            Catch ex As Exception
                ErrorLog.WriteTextLog(ex.Message & " - Custom Button Insertion Failed", ex.StackTrace)
                Return False
            End Try
        End If

    End Function

    ''' <summary>
    ''' Function to Delete POS Entry
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Sub DeletePOS()

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlCmd As New SqlCommand(Query.deleteCustomButtonPOS, connection)
                    sqlCmd.ExecuteNonQuery()
                End Using
                connection.Close()
            End Using
        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Custom Button Deletion Failed", ex.StackTrace)
        End Try

    End Sub

    ''' <summary>
    ''' Function Returns Maximum POS Entry Number in DB
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared Function GetMaxNumberOfPOS() As Integer

        Dim maxNumber As Integer = Nothing

        Try
            Using connection As New SqlConnection(GetConnectionString())
                connection.Open()
                Using sqlCmd As New SqlCommand(Query.selectMaxEntryPOS, connection)
                    maxNumber = sqlCmd.ExecuteScalar()
                    maxNumber += 1
                End Using
                connection.Close()
            End Using
            Return maxNumber

        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message & " - Error in getting the maximum Number", ex.StackTrace)
            Return 999
        End Try

    End Function

    '''' <summary>
    '''' Function Saves Application Connection String
    '''' </summary>
    '''' <param name="connStr"></param>
    '''' <remarks></remarks>
    'Public Shared Sub SaveConnectionString(ByVal connStr As String)
    '    Dim fileMap As New ExeConfigurationFileMap
    '    Dim config As System.Configuration.Configuration
    '    Dim exePath As String

    '    Try

    '        'Sets the installation path where app.config is located
    '        exePath = strInstallationPath & "SmartShopping.Data.dll.config"

    '        fileMap.ExeConfigFilename = exePath

    '        'To open the exe file where the connection string is present
    '        config = ConfigurationManager.OpenMappedExeConfiguration _
    '            (fileMap, ConfigurationUserLevel.None)

    '        'Sets the Value in the config file
    '        config.AppSettings.Settings.Item("ConnString").Value = connStr

    '        config.Save()

    '    Catch ex As Exception
    '        ErrorLog.WriteTextLog(ex.Message & " - Failed to Save Connection String", ex.StackTrace)
    '    End Try
    'End Sub

    '''' <summary>
    '''' Functions Sets Connection String for Application
    '''' </summary>
    '''' <remarks></remarks>
    'Public Shared Sub SetConnectionString()

    '    Try
    '        gConnectionString = ConfigurationManager.AppSettings("ConnString")
    '    Catch ex As Exception
    '        ErrorLog.WriteTextLog(ex.Message & " - Failed to Set Connection String", ex.StackTrace)
    '    End Try

    'End Sub

End Class
#End Region