﻿#Region "File Header"
' ----------------------------------------------------------------------------------------
' File Name          :   AppInstaller.vb
' Copyright          :   Mindfire Solutions, Copyright statement. All right reserved
' File Description   :   1.Opens 'DB Setting' form and takes valid database settings as user input
'                        2.Rollbacks the installation if error occurs  
'
' Date Created       :   11/22/2012
' Date Modified      :   
' ----------------------------------------------------------------------------------------
#End Region

#Region "Namespaces"
Imports System.ComponentModel
Imports System.Configuration.Install
Imports System.Windows.Forms
Imports SmartShopping.Data
Imports Microsoft.Win32
#End Region

''' <summary>
''' Class used for Custom Install
''' </summary>
''' <remarks></remarks>
Public Class AppInstaller

    ''' <summary>
    ''' Functions Overrides Default Constructor
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()

        'This call is required by the Component Designer.
        InitializeComponent()

        'Add initialization code after the call to InitializeComponent

    End Sub

    ''' <summary>
    ''' Function Executes at the time of installation
    ''' </summary>
    ''' <param name="stateSaver"></param>
    ''' <remarks></remarks>
    Public Overrides Sub Install(ByVal stateSaver As System.Collections.IDictionary)

        MyBase.Install(stateSaver)

    End Sub

    ''' <summary>
    ''' Function Opens the DBSetting form at the time of installation
    ''' </summary>
    ''' <param name="savedState"></param>
    ''' <remarks></remarks>
    Public Overrides Sub Commit(ByVal savedState As System.Collections.IDictionary)

        Dim objWinFrmDbSettings As New WinFormDbSettings 'Object for DBSettings form

        strInstallationPath = Me.Context.Parameters.Item("DIR")

        'If user cancels the DBSettings form then a rollback occurs
        If objWinFrmDbSettings.ShowDialog = DialogResult.Cancel Then

            Throw New Exception("This will Rollback the installation.")

        End If



        MyBase.Commit(savedState)

    End Sub

    ''' <summary>
    ''' Function Uninstalls the ReportTableGenerator
    ''' </summary>
    ''' <param name="savedState"></param>
    ''' <remarks></remarks>
    Public Overrides Sub Uninstall(ByVal savedState As System.Collections.IDictionary)

        Dim objDbOperation As New DbOperations()
        Dim registryKey As RegistryKey = Nothing

        registryKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\SmartShopping", False)

        If Not registryKey Is Nothing Then

            Dim server As String = Convert.ToString(registryKey.GetValue("ServerName"))
            Dim db As String = Convert.ToString(registryKey.GetValue("DbName"))
            Dim user As String = Convert.ToString(registryKey.GetValue("UserName"))
            Dim pwd As String = Convert.ToString(registryKey.GetValue("PassWord"))

            gConnectionString = "Data Source=" & server & ";Initial Catalog=" & db & _
                        ";User ID=" & user & ";Password=" & pwd & ";"

        End If

        objDbOperation.DeletePOS()

        MyBase.Uninstall(savedState)

    End Sub

    ''' <summary>
    ''' Functions Rollbacks the installation
    ''' </summary>
    ''' <param name="savedState"></param>
    ''' <remarks></remarks>
    Public Overrides Sub Rollback(ByVal savedState As System.Collections.IDictionary)

        MyBase.Rollback(savedState)

    End Sub


End Class
