﻿#Region "File Header"
'-----------------------------------------------------------------------
' File Name         :   WinFormDbSettings.vb
' File Description  :   1.User Database Settings Input UI
'                       2.Create Custom POS Button
'
' Date Created      :   28th September 2012
' Date Modified     :
'-----------------------------------------------------------------------
#End Region

#Region "Namespaces"
Imports System
Imports System.Windows.Forms
Imports System.Data.SqlClient
Imports SmartShopping.Data
Imports Microsoft.Win32
#End Region

#Region "WinFormDbSettings Class"
Public Class WinFormDbSettings

    ''' <summary>
    ''' Function Handles WinForm DbSettings Load Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub WinFormDbSettings_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'lblFooter.Text = "Mindfire Solutions" & vbNewLine & "India"
        txtServerName.Focus()
    End Sub

    ''' <summary>
    ''' Function Handles Button Reset Click Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReset.Click

        Dim textboxList As New List(Of TextBox)
        textboxList.Add(txtServerName)
        textboxList.Add(txtDbName)
        textboxList.Add(txtUserName)
        textboxList.Add(txtPassword)

        'Function Call to Reset TextBoxes
        ResetTextBoxes(textboxList)

    End Sub

    ''' <summary>
    ''' Function Handles Button Test Connection Click Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnTest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTest.Click

        Dim connectionString As String = BuildConnectionString()

        If Not String.IsNullOrEmpty(connectionString) Then
            If SettingRMS.TestDbConnection(connectionString) Then
                MessageBox.Show("Database Connection Successful !")
            Else
                txtServerName.Focus()
            End If
        End If

    End Sub

    ''' <summary>
    ''' Function Handles Button Create Custom POS Click Event
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnCreatePOS_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCreatePOS.Click

        Dim connectionString As String = BuildConnectionString()

        If Not String.IsNullOrEmpty(connectionString) Then
            Dim RmsSettings As New SettingRMS()

            If SettingRMS.TestDbConnection(connectionString) Then
                SaveConnStringInRegistry()
                RmsSettings.CreateCustomPOS()

                If DbOperations.CreateStoreImagesTable() Then
                    ErrorLog.WriteTextLog("Store Images Table Creation", " Success")
                Else
                    ErrorLog.WriteTextLog("Store Images Table Creation", "Failed")
                End If

                Me.DialogResult = Windows.Forms.DialogResult.OK
            Else
                txtServerName.Focus()

                Me.DialogResult = Windows.Forms.DialogResult.None
            End If

        End If

    End Sub

    ''' <summary>
    ''' Function Builds the Connection String
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function BuildConnectionString() As String

        'Checking TextBoxes for Null or Empty
        If Not (String.IsNullOrEmpty(txtServerName.Text) Or _
                String.IsNullOrEmpty(txtDbName.Text) Or _
                String.IsNullOrEmpty(txtUserName.Text) Or _
                String.IsNullOrEmpty(txtPassword.Text)) Then

            'Building Connection String
            Dim connStr As New SqlConnectionStringBuilder()
            connStr.DataSource = txtServerName.Text.Trim()
            connStr.InitialCatalog = txtDbName.Text.Trim()
            connStr.UserID = txtUserName.Text.Trim()
            connStr.Password = txtPassword.Text.Trim()
            'connStr.IntegratedSecurity = True
            gConnectionString = connStr.ConnectionString
            Return connStr.ConnectionString

        Else
            txtServerName.Focus()
            Return ""
        End If

    End Function

    ''' <summary>
    ''' Functions Resets the TextBox Values and Negate the ReadOnly Property
    ''' </summary>
    ''' <param name="textboxList"></param>
    ''' <remarks></remarks>
    Private Sub ResetTextBoxes(ByVal textboxList As List(Of TextBox))

        'Setting the TextBoxes Blank and Negating ReadOnly Property
        For Each objTextBox As TextBox In textboxList
            objTextBox.Text = ""
            objTextBox.ReadOnly = False
        Next

        'Keeping First TextBox to Focus
        textboxList(0).Focus()

    End Sub

    ''' <summary>
    ''' Saves the connection string in registry. 
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SaveConnStringInRegistry()

        Dim registryKey As RegistryKey = Nothing

        Try

            registryKey = Registry.LocalMachine.CreateSubKey("SOFTWARE\SmartShopping")

            If Not registryKey Is Nothing Then

                registryKey.SetValue("ServerName", txtServerName.Text.Trim())
                registryKey.SetValue("DbName", txtDbName.Text.Trim())
                registryKey.SetValue("UserName", txtUserName.Text.Trim())
                registryKey.SetValue("PassWord", txtPassword.Text.Trim())

            End If

            registryKey.Close()

        Catch ex As Exception
            ErrorLog.WriteTextLog(ex.Message, ex.StackTrace)
        End Try

    End Sub

End Class
#End Region